/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        BaseException: an exception for any kind of CREAM's fault
	
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: BaseException.h,v 1.3.4.3 2007/03/29 10:39:31 adorigo Exp $
*/

#ifndef _GLITE_CE_CREAM_CLIENT_API_CREAMEX_BASEEX_H__
#define _GLITE_CE_CREAM_CLIENT_API_CREAMEX_BASEEX_H__

#include<exception>
#include<string>
#include<ctime>
#include <cstring> // for memset
#include <boost/format.hpp>

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace cream_exceptions {
	
	class BaseException : public std::exception {
	  
	  std::string  m_methodname;
	  std::string  m_errorcode;
	  std::string  m_description;
	  std::string  m_faultcause;
	  time_t       m_timestamp;
	  
	public:

	  BaseException(const std::string& mn,
			const std::string& ec,
			const std::string& des,
			const std::string& fc,
			const time_t& t) 
	    : m_methodname(mn), m_errorcode(ec), m_description(des), m_faultcause(fc), m_timestamp(t) {}

	  virtual ~BaseException() throw() {}

	  virtual const char* what() const throw() { 
	    char timestamp[40]; 
	    struct tm TSTAMP;
	    memset((void*)timestamp, 0, 40); 
	    localtime_r(&m_timestamp, &TSTAMP);
	    strftime(timestamp, sizeof(timestamp), "%a %d %b %Y %T", &TSTAMP);

	    std::string whatinfo = boost::str( boost::format("MethodName=[%s] ErrorCode=[%s] Description=[%s] FaultCause=[%s] Timestamp=[%s]") % m_methodname % m_errorcode % m_description % m_faultcause % timestamp );

/*	    return (std::string("MethodName=[")+m_methodname
		    +"] ErrorCode=["+m_errorcode
		    +"] Description=["+m_description
		    +"] FaultCause=["+m_faultcause+"] Timestamp=["+std::string(timestamp)+"]");*/
	    return whatinfo.c_str();
	  }
	  virtual const char* getMethodName() const { return m_methodname.c_str(); }
	  virtual const char* getErrorCode() const { return m_errorcode.c_str(); }
	  virtual const char* getDescription() const { return m_description.c_str(); }
	  virtual const char* getFaultCause() const { return m_faultcause.c_str(); }
	  virtual time_t      getTimeStamp() const { return m_timestamp; }
	};
      }
    }
  }
}

#endif
