/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEUrl: tools to nicely parse a CE Url

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEUrl.h,v 1.8.4.4 2007/03/22 15:46:02 adorigo Exp $
*/
#ifndef _GLITE_CE_CREAM_CLIENT_API_UTIL_CEURL_H_
#define _GLITE_CE_CREAM_CLIENT_API_UTIL_CEURL_H_

#include <vector>
#include <string>
#include <map>
#include "glite/ce/cream-client-api-c/ceid_syntax_ex.h"
#include "boost/regex.hpp"

namespace glite {
  namespace ce {
    namespace cream_client_api {
      namespace util {

	namespace CEUrl {

	  static const boost::regex s_ceid_pattern("^([^:]+)(:[0-9]+)?/cream-([^-]+)-([^-]+)");
	  static const boost::regex s_jceid_pattern("^(https?)://([^:]+)(:[0-9]+)?/(.+)");

	  void parseCEID(const std::string& ceid, std::vector<std::string>&,
	                 const std::string& default_tcpport = "8443")
	    throw(glite::ce::cream_client_api::util::CEUrl::ceid_syntax_ex&);

	  void parseJobID(const std::string& jid, std::vector<std::string>&,
	                  const std::string& default_tcpport = "8443")
	    throw(glite::ce::cream_client_api::util::CEUrl::ceid_syntax_ex&);

	  void organise_by_endpoint(const std::vector<std::string>&,
				    std::map<std::string, std::vector<std::string> >&,
				    const char*)
	    throw(ceid_syntax_ex&);

	  void extractHostPort(const std::string& endpoint, std::string& host, int& port) throw();

	  std::string extractEndpointFromURL(const std::string& url) throw();
	}
      }
    }
  }
}

#endif
