/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        certUtil: utilities to extract useful info from a proxy certificate
	
        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: certUtil.h,v 1.4.2.5 2007/05/03 13:40:28 adorigo Exp $
*/

#ifndef _GLITE_CE_CREAM_CLIENT_API_UTIL_CERTUTIL_H_
#define _GLITE_CE_CREAM_CLIENT_API_UTIL_CERTUTIL_H_
 
#include "glite/ce/cream-client-api-c/auth_ex.h"
#include <string>
#include <glite/security/voms/voms_api.h>
#include <openssl/pem.h>

namespace glite {
namespace ce {
namespace cream_client_api {

  class helper_BIOnew {
    BIO *in;
  public:
    helper_BIOnew(BIO* infile) : in(infile) {}
    ~helper_BIOnew() { 
      if(in) {
	BIO_set_close(in, BIO_CLOSE);    
      //BIO_close(in);
	BIO_free(in);
      } 
    }
  };
  
  class helper_X509new {
    X509* x;
  public:
    helper_X509new(X509* xin) : x(xin) {}
    ~helper_X509new() { if(x) X509_free(x); }
  };
  
  class helper_skX509new {
    STACK_OF(X509) *sk;
  public:
    helper_skX509new(STACK_OF(X509)* x) : sk(x) {}
    ~helper_skX509new() {
      if( sk ) {
	sk_X509_pop_free( sk, X509_free );
	//sk_X509_free( sk ); // FIXME without this seems to be some mem leak
	// but with this some invalid memory read occurs
      }
    }
  };

  /**
     This namespace provides useful proxy-certificate related functions
  */
  namespace certUtil {
    X509* read_BIO( const std::string& pxfile ) 
      throw( glite::ce::cream_client_api::soap_proxy::auth_ex& );

    /**
       DEPRECATED
       Returns the Issuer name contained in a proxy certificate
       \param proxyfile The file containing the certificate
       \return A string representing the DN in X509 format (like /C=IT/O=INFN/OU=Personal Certificate/L=Padova/CN=Your Name/CN=proxy)
    */
    std::string getDN(const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);

    std::string getDNFQAN(const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
    
    /**
       Returns the Issuer name contained in a proxy certificate
       \param proxyfile The file containing the certificate
       \return A string representing the DN in X509 format (like /C=IT/O=INFN/OU=Personal Certificate/L=Padova/CN=Your Name/CN=proxy)
    */
    std::string getCertIssuer(const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
        
    /**
       Returns the Subject name contained in a proxy certificate
       \param proxyfile The file containing the certificate
       \return A string representing the DN in X509 format (like /C=IT/O=INFN/OU=Personal Certificate/L=Padova/CN=Your Name/CN=proxy)
    */
    std::string getCertSubj(const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
            
    /**
       Returns the Virtual Organization name contained in a voms-generated proxy certificate
       \param proxyfile The file containing the certificate
       \return A string representing the name of the VO
    */
    std::string getVOName( const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
    
    /**
       Returns the number of seconds left before the proxy certificate expiration
       \param proxyfile The file containing the certificate
       \return A time_t value
    */
    const long int getProxyTimeLeft( const std::string& proxyfile )
      throw (glite::ce::cream_client_api::soap_proxy::auth_ex&);
    
    /**
       This method is conceived only for "internal" usage. The user is not
       supposed to be interested in.
    */	    
    X509* getX509Fromfile(const std::string& proxyfile)
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
    
    /**
       This method is conceived only for "internal" usage. The user is not
       supposed to be interested in.
    */
    STACK_OF(X509) *load_chain(char *certfile) throw();
    
    /**
       This method is conceived only for "internal" usage. The user is not
       supposed to be interested in.
    */
    void getFQANs( const std::string&, std::vector<std::string>& )
      throw(glite::ce::cream_client_api::soap_proxy::auth_ex&);
  };
  
 }
 }
}

#endif
